@@ -1,7 +1,7 @@
 /*
  *  magicfilter.c
  *
- *  Copyright  1993-95 H. Peter Anvin
+ *  Copyright  1993-96 H. Peter Anvin
  *
  *  "Magic" default filter for lpr/lpd which attempt to identify
  *  the type of file and filter it accordingly
@@ -714,7 +714,7 @@ void act_fpipe(char *command)
 	}
       else if ( pid == 0 )		/* Child process */
 	{
-	  putenv(tmpbuf);		/* Put filename in $FILE */
+	  putenv(tmpbuf);		/* Store name in $FILE */
 	  outfd = open(tmpbuf+5, O_RDONLY);
 	  dup2(outfd,fileno(stdin)); /* Redirect stdin */
 	  dup2(outpipefd[1],fileno(stdout)); /* Redirect stdout */
@@ -804,6 +804,30 @@ int matches(char *block, char *pattern, char *mask, int length)
   return 1;
 }
 
+/* ------------------------------------------------------------------------- *
+ *  envset(var,value)
+ * 
+ * Inserts a key value into the environment
+ * ------------------------------------------------------------------------- */
+
+void envset(char *var, char *value)
+{
+  char *buf;
+
+  if ( !(buf = malloc(strlen(var)+strlen(value)+2)) )
+    {
+      fprintf(stderr, "%s: out of memory\n", ERR_NAME);
+      exit(1);
+    }
+  sprintf(buf, "%s=%s", var, value);
+  if ( putenv(buf) )
+    {
+      perror(ERR_NAME);
+      exit(1);
+    }
+  if ( debug_flag )
+    fprintf(stderr, "%s\n", buf);
+}
  
 /* ------------------------------------------------------------------------- *
  *  main(argc,argv)
@@ -819,9 +843,6 @@ main(int argc, char *argv[])
   const struct datatype *dt;	/* Data type searched */
   char *user = "unknown";	/* User name */
   char *host = "unknown";	/* Host name */
-  char *indent;			/* Indentation (-i option) */
-  char *lp_user, *lp_host; 	/* For setenv command */
-  char *lp_indent;
 
   program = argv[0];		/* Make available for error messages */
 
@@ -831,15 +852,37 @@ main(int argc, char *argv[])
       exit(1);
     }
 
-  /* Clear these strings from environment
-     This actually put it in as null strings, but unsetenv() is
-     apparently not portable.  This might be a good use for
-     autoconf. */
+  /* Clear these strings from environment; if we have unsetenv use it,
+     otherwise set to blank strings */
 
+#ifdef HAVE_UNSETENV
+  unsetenv("LPUSER");
+  unsetenv("LPHOST");
+  unsetenv("LPJOB");
+  unsetenv("FILE");
+  unsetenv("PRINTER");
+  unsetenv("LPFORMAT");
+  unsetenv("LPCLASS");
+  unsetenv("LPACCT");
+  unsetenv("LPCOPIES");
+  unsetenv("LPQUEUE");
+  unsetenv("ZOPT");
+  unsetenv("BANNERNAME");
+#else
   putenv("LPUSER=");
   putenv("LPHOST=");
-  putenv("LPINDENT=0");		/* Default to zero */
+  putenv("LPJOB=");
   putenv("FILE=");
+  putenv("PRINTER=");
+  putenv("LPFORMAT=");
+  putenv("LPCLASS=");
+  putenv("LPACCT=");
+  putenv("LPCOPIES=");
+  putenv("LPQUEUE=");
+  putenv("ZOPT=");
+  putenv("BANNERNAME=");
+#endif
+  putenv("LPINDENT=0");		/* Default to zero */
 
   for ( i = 1 ; i < argc ; i++ ) /* Options passed by lpd */
     {
@@ -847,39 +890,36 @@ main(int argc, char *argv[])
 	{
 	  if ( strcmp(argv[i],"-c") == 0 )
 	    do_cat = 1;
-	  else if ( strcmp(argv[i],"-n") == 0 && i < argc-1 )
+	  else if ( strncmp(argv[i],"-n",2) == 0 )
 	    {
-	      user = argv[++i];
-	      if ( (lp_user = malloc(strlen(user)+8)) == NULL )
-		{
-		  fprintf(stderr, "%s: Out of memory\n", argv[1]);
-		  exit(1);
-		}
-	      sprintf(lp_user, "LPUSER=%s", user);
-	      putenv(lp_user);
+	      user = (argv[i][2] || i == argc-1) ? argv[i]+2 : argv[++i];
+	      envset("LPUSER", user);
 	    }
-	  else if ( strcmp(argv[i],"-h") == 0 && i < argc-1 )
+	  else if ( strncmp(argv[i],"-h",2) == 0 )
 	    {
-	      host = argv[++i];
-	      if ( (lp_host = malloc(strlen(host)+8)) == NULL )
-		{
-		  fprintf(stderr, "%s: Out of memory\n", argv[1]);
-		  exit(1);
-		}
-	      sprintf(lp_host, "LPHOST=%s", host);
-	      putenv(lp_host);
+	      host = (argv[i][2] || i == argc-1) ? argv[i]+2 : argv[++i];
+	      envset("LPHOST", host);
 	    }
 	  else if ( strncmp(argv[i],"-i",2) == 0 )
-	    {
-	      indent = argv[i]+2;
-	      if ( (lp_indent = malloc(strlen(indent)+10)) == NULL )
-		{
-		  fprintf(stderr, "%s: Out of memory\n", argv[1]);
-		  exit(1);
-		}
-	      sprintf(lp_indent, "LPINDENT=%s", indent);
-	      putenv(lp_indent);
-	    }
+	    envset("LPINDENT", argv[i]+2);
+	  else if ( strncmp(argv[i],"-C",2) == 0 )
+	    envset("LPCLASS", argv[i]+2);
+	  else if ( strncmp(argv[i],"-F",2) == 0 )
+	    envset("LPFORMAT", argv[i]+2);
+	  else if ( strncmp(argv[i],"-J",2) == 0 )
+	    envset("LPJOB", argv[i]+2);
+	  else if ( strncmp(argv[i],"-K",2) == 0 )
+	    envset("LPCOPIES", argv[i]+2);
+	  else if ( strncmp(argv[i],"-L",2) == 0 )
+	    envset("BANNERNAME", argv[i]+2);
+	  else if ( strncmp(argv[i],"-P",2) == 0 )
+	    envset("PRINTER", argv[i]+2);
+	  else if ( strncmp(argv[i],"-Q",2) == 0 )
+	    envset("LPQUEUE", argv[i]+2);
+	  else if ( strncmp(argv[i],"-R",2) == 0 )
+	    envset("LPACCT", argv[i]+2);
+	  else if ( strncmp(argv[i],"-Z",2) == 0 )
+	    envset("ZOPT", argv[i]+2);
 	  else if ( strcmp(argv[i], "--debug") == 0 )
 	    debug_flag = 1;
 	  /* Ignore other options passed by lpd */
